//DirectInput code

//Includes
//regular headers
#include <ostream>
#include <iostream>
//Created Headers
#include "DXInput.h"

//namespaces
using namespace std;

//macro for detecting mouse button presses
#define BUTTON_DOWN(obj, button) (obj.rgbButtons[button] & 0x80)

//dinput objects
LPDIRECTINPUT8 dinput; //main input object
LPDIRECTINPUTDEVICE8 dimouse; //mouse object
LPDIRECTINPUTDEVICE8 dikeyboard; //keyboard object
DIMOUSESTATE mouse_state; //struct that holds mouse information (movement, buttons, etc.)

//keyboard array
char keys[256];

//intialize main directinput objects
int Init_DirectInput(HWND hwnd)
{
	HRESULT result;
	
	//initialize Directinput object
	result=DirectInput8Create(GetModuleHandle(NULL),DIRECTINPUT_VERSION,IID_IDirectInput8,(void**)&dinput,NULL);
	if(result!=DI_OK)
	{
		cerr<<"DirectInput8Create failed \n";
		return 0;}

	//create mouse object
	result = dinput->CreateDevice(GUID_SysMouse,&dimouse,NULL);
	if(result!=DI_OK)
	{
		cerr<<"DirectInput CreateDevice Mouse failed\n";
		return 0;}

	//create keyboard object
	result = dinput->CreateDevice(GUID_SysKeyboard,&dikeyboard,NULL);
	if(result!=DI_OK)
	{
		cerr<<"DirectInput CreateDevice Keyboard failed\n";
		return 0;}

	cerr<<"DirectInput Initialized\n";
	return 1;
}

//intialize mouse
int Init_Mouse(HWND hwnd)
{
	HRESULT result;

	//set data format for mouse input
	result=dimouse->SetDataFormat(&c_dfDIMouse);
	if(result!=DI_OK)
	{
		cerr<<"Init_mouse SetDataFormat failed\n";
		return 0;}

	//set cooperative level for mouse
	result=dimouse->SetCooperativeLevel(hwnd,DISCL_EXCLUSIVE | DISCL_FOREGROUND);
	if(result!=DI_OK)
	{
		cerr<<"Init_Mouse SetCooperativeLevel failed\n";
		return 0;}

	//acquire mouse
	result=dimouse->Acquire();
	if(result!=DI_OK)
	{
		cerr<<"Init_mouse Acquire failed\n";
		return 0;}

	cerr<<"Mouse Initialized\n";
	return 1;
}

//mouse x movement
int Mouse_X()
{
	return mouse_state.lX;
}

//mouse y movement
int Mouse_Y()
{
	return mouse_state.lY;
}

//is button down?
int Mouse_Button(int button)
{
	return BUTTON_DOWN(mouse_state, button);
}

//get mouse movement and button information
void Poll_Mouse()
{
	dimouse->GetDeviceState(sizeof(mouse_state),(LPVOID)&mouse_state);
}

//release mouse objects
void Kill_Mouse()
{
	if(dimouse!=NULL)
	{
		dimouse->Unacquire();
		dimouse->Release();
		dimouse=NULL;
	}
	cerr<<"Mouse Object Released\n";
}

//set up keyboard objects
int Init_Keyboard(HWND hwnd)
{
	HRESULT result;
	
	//set data format for keyboard
	result=dikeyboard->SetDataFormat(&c_dfDIKeyboard);
	if(result!=DI_OK)
	{
		cerr<<"Init_Keyboard SetDataFormat failed\n";
		return 0;}

	//set cooperative level
	result=dikeyboard->SetCooperativeLevel(hwnd,DISCL_NONEXCLUSIVE | DISCL_FOREGROUND);
	if(result!=DI_OK)
	{
		cerr<<"Init_Keyboard SetCooperativeLevel failed\n";
		return 0;}

	//acquire the keyboard
	result=dikeyboard->Acquire();
	if(result!=DI_OK)
	{
		cerr<<"Init_Keyboard Acquire failed\n";
		return 0;}

	cerr<<"Keyboard Initialized\n";
	return 1;
}

//fill keyboard array with button information
void Poll_Keyboard()
{
	dikeyboard->GetDeviceState(sizeof(keys),(LPVOID)&keys);
}

//is key down?
int Key_Down(int key)
{
	return (keys[key] & 0x80);
}

//release keyboard objects
void Kill_Keyboard()
{
	if(dikeyboard!=NULL)
	{
		dikeyboard->Unacquire();
		dikeyboard->Release();
		dikeyboard=NULL;
	}
	cerr<<"Keyboard objects released\n";
}



