//Main Game Source
//Ted the Bastard Sloth Saves Some Lake
//Programming by Adam Sepanski for the Charlotte Game Development Group
//Created on 8/20/06

//Includes
//regular headers
#include <ostream>
#include <iostream>
//Created Headers
#include "Game.h"

//namespaces
using namespace std;

//constants
const int PLAYER_VELOCITY = 10;//players movement speed (pixels per second)
const int PLAYER_FRAMES_OF_ANIMATION=4;
//used only to keep up functionality until we have to redo these
const int KID_HORIZONTAL_BOUNDING_REDUCTION=1;
const int AMMO_VELOCITY = 15;//ammo speed
const float GRAVITY_ACCELERATION =0.2f;//acceleration due to gravity
const int AMMO_MAXIMUM_COUNT = 50;//maximum amount of ammo available
const bool AMMO_X_MOTION = false;//flag to set ammo x motion on or off
const int TRASH_Y_VELOCITY=3;//y velocity of trash
const int EXPLOSION_MAXIMUM_COUNT=20;//maximum amount of explosions
const int EXPLOSION_LIFE=50;//lifetime of explosions (in frames)
const int UFO_SPEED=4;//speed of the ufo
const int UFO_TRASH_DELAY=1000;//delay of trash to fall in ms (this is the minimum time, a randomizer is added)
const int UFO_FRAMES_OF_ANIMATION=4;
const int PLANE_SPEED=3;//speed of the plane
const int PLANE_TRASH_DELAY=3000;
const int PLANE_FLYOFF_DELAY=150;
const int PLANE_FRAMES_OF_ANIMATION=4;
const int HELICOPTER_SPEED=2;
const int HELICOPTER_TRASH_DELAY=1000;
const int HELICOPTER_FRAMES_OF_ANIMATION=4;
const int CESSNA_SPEED=2;
const int CESSNA_TRASH_DELAY=1750;
const int CESSNA_FLYOFF_DELAY=60;
const int CESSNA_FRAMES_OF_ANIMATION=4;
//these are ugly hacks to make up for the fact that I didn't use a class.  This will be the id used
//when calling the firetrash() function to ensure that the trash drops from the correct position (ie center
//for ufo, bottom left of plane when moving left, etc.)
const int TRASH_FUNCTION_UFO=1;
const int TRASH_FUNCTION_PLANE=2;
const int TRASH_FUNCTION_HELICOPTER=3;
const int TRASH_FUNCTION_CESSNA=4;

const int MAX_LAKEHEALTH=10;
const int MAX_PLAYERHEALTH=1000;


//Game States
const int GAME_RUNNING=1; //Game is being played
const int GAME_MENU=2; //game is in main menu
const int GAME_LEVEL_LOAD=3;//sets up the enemies and winning conditions necessary for the current level
const int GAME_LEVEL_RESET=4;//resets the variables for a new level

//Surfaces
LPDIRECT3DSURFACE9 background_surface;//background surface

//Sprites
SPRITE_PLAYER SpritePlayer;//main character
SPRITE_AMMO SpriteAmmo[AMMO_MAXIMUM_COUNT];//ammunition
SPRITE_MISC SpriteExplosion[EXPLOSION_MAXIMUM_COUNT];//explosions
SPRITE_ENEMY SpriteUFO;//UFO that drops trash
SPRITE_ENEMY SpritePlane;
SPRITE_ENEMY SpriteHelicopter;
SPRITE_ENEMY SpriteCessna;

//Sounds
CSound *sound_1;

//Timing variable
//frame time will be taken by subtracting 
long FrameTimeCurrent=GetTickCount();//tickcount starting on current frame
long FrameTimePrevious=GetTickCount();//tickcount starting on previous frame
float FrameTime=0.0f;//time of each frame in seconds
float FramesPerSecond=0.0f;//current FPS
long start=0;
long counter=0;//for testing frame count

//Keyboard variables
//these are necessary to detect a key press not key down
bool RightKeyPressed=false;
bool LeftKeyPressed=false;
bool UpKeyPressed=false;
bool DownKeyPressed=false;
bool AKeyPressed=false;
bool SKeyPressed=false;
bool DKeyPressed=false;
bool WKeyPressed=false;
bool RControlKeyPressed=false;
bool FKeyPressed=false;
bool EscKeyPressed=false;
bool EnterKeyPressed=false;
bool PlusKeyPressed=false;
bool MinusKeyPressed=false;

//Misc.
D3DXVECTOR3 position(0,0,0);//vector that holds sprite position for sprite handler
int GameState=0;//holds the current game status
int AmmoCount=0;//what ammo is ready to fire

int ExplosionCount=0;//what explosion is ready to display
RECT rectTextHealth;//rectangle for position of the text displaying health
RECT rectTextLakeHealth;//rectange for position of the text displaying the lake health
RECT rectTextLevel;
int LakeHealth=MAX_LAKEHEALTH;//health of the lake (how much trash can drop on it)
int GameLevel=1;//level player is currently on
unsigned long int LevelTime=0;//amount of time level has been loaded
unsigned long int LevelWinTime=0;//time needed for level win

//////////////////////////////////////
////////////Functions/////////////////

//initializes game
int Game_Init(HWND hwnd)
{
	//random number seed
	srand(time(NULL));

	//load background images onto surfaces
	background_surface=LoadSurface("springbackground.png",0);

	//load player sprite images onto texture, with pink as transparent
	SpritePlayer.texture=LoadTexture("tedframes.png",D3DCOLOR_XRGB(255,0,255));//main character
	//only load the ammo surface into the first one in array, save space
	SpriteAmmo[0].texture=LoadTexture("ammo.png",D3DCOLOR_XRGB(255,0,255));//ammo
	//load explosion image onto texture
	SpriteExplosion[0].texture=LoadTexture("explosion.png",D3DCOLOR_XRGB(255,0,255));
	//load ufo onto texture
	SpriteUFO.texture=LoadTexture("ufoframes.png",D3DCOLOR_XRGB(255,0,255));
	//load trash image onto texture
	SpriteUFO.Trash[0].texture=LoadTexture("banana peelreal.png",D3DCOLOR_XRGB(255,0,255));//banana peel trash
	//load plane onto texture
	SpritePlane.texture=LoadTexture("planeframes.png",D3DCOLOR_XRGB(255,0,255));
	//load the plane's trash onto a texture
	SpritePlane.Trash[0].texture=LoadTexture("dead fish.png",D3DCOLOR_XRGB(255,0,255));//drum trash
	//load the helicopter onto a texture
	SpriteHelicopter.texture=LoadTexture("helicopterframes.png",D3DCOLOR_XRGB(255,0,255));
	//load heli's trash
	SpriteHelicopter.Trash[0].texture=LoadTexture("soda can.png",D3DCOLOR_XRGB(255,0,255));//keen trash
	//load the cessna onto a texture
	SpriteCessna.texture=LoadTexture("cessnaframes.png",D3DCOLOR_XRGB(255,0,255));
	//load cessna's trash
	SpriteCessna.Trash[0].texture=LoadTexture("sneaker.png",D3DCOLOR_XRGB(255,0,255));//fish trash

	//set player properties
	SpritePlayer.x=50;
	SpritePlayer.y=625;
	SpritePlayer.width=72;
	SpritePlayer.height=124;
	SpritePlayer.vx=PLAYER_VELOCITY;
	SpritePlayer.vy=0;
	SpritePlayer.animdelay=80;
	SpritePlayer.animcount=GetTickCount();
	SpritePlayer.framecount=0;
	SpritePlayer.facingright=true;
	
	SpritePlayer.health=MAX_PLAYERHEALTH;
	SpritePlayer.visible=true;

	//set ammo properties
	for(int index=0;index<AMMO_MAXIMUM_COUNT;index++)
	{
		SpriteAmmo[index].x=0;
		SpriteAmmo[index].y=0;
		SpriteAmmo[index].width=24;
		SpriteAmmo[index].height=24;
		SpriteAmmo[index].vx=0;
		SpriteAmmo[index].vy=AMMO_VELOCITY;
		SpriteAmmo[index].visible=false;
	}

	//set explosion properties
	for(int index=0;index<EXPLOSION_MAXIMUM_COUNT;index++)
	{
		SpriteExplosion[index].x=0;
		SpriteExplosion[index].y=SCREEN_HEIGHT-45;
		SpriteExplosion[index].width=124;
		SpriteExplosion[index].height=36;
		SpriteExplosion[index].visible=false;
	}

	//set ufo properties
	SpriteUFO.width=144;
	SpriteUFO.height=56;
	SpriteUFO.x=800;
	SpriteUFO.y=10;
	SpriteUFO.vx=-UFO_SPEED;
	SpriteUFO.alive=false;
	SpriteUFO.visible=false;
	SpriteUFO.animdelay=80;
	SpriteUFO.animcount=GetTickCount();
	SpriteUFO.framecount=0;
	SpriteUFO.facingright=false;
	//set trash properties
	SpriteUFO.TrashCount=0;
	SpriteUFO.TrashDelay=0;
	SpriteUFO.TrashTime=GetTickCount();
	for(int index=0;index<TRASH_MAXIMUM_COUNT;index++)
	{
		SpriteUFO.Trash[index].x=700;
		SpriteUFO.Trash[index].y=0;
		SpriteUFO.Trash[index].width=64;
		SpriteUFO.Trash[index].height=48;
		SpriteUFO.Trash[index].vx=0;
		SpriteUFO.Trash[index].vy=TRASH_Y_VELOCITY;
		SpriteUFO.Trash[index].visible=false;
	}

	//set plane properties
	SpritePlane.width=136;
	SpritePlane.height=104;
	SpritePlane.x=10;
	SpritePlane.y=10;
	SpritePlane.vx=PLANE_SPEED;
	SpritePlane.alive=false;
	SpritePlane.visible=false;
	SpritePlane.animdelay=80;
	SpritePlane.animcount=GetTickCount();
	SpritePlane.framecount=0;
	SpritePlane.facingright=true;
	//set trash properties
	SpritePlane.TrashCount=0;
	SpritePlane.TrashDelay=0;
	SpritePlane.TrashTime=GetTickCount();
	SpritePlane.flyoffcount=0;
	SpritePlane.flyoffdelay=PLANE_FLYOFF_DELAY;
	for(int index=0;index<TRASH_MAXIMUM_COUNT;index++)
	{
		SpritePlane.Trash[index].x=700;
		SpritePlane.Trash[index].y=0;
		SpritePlane.Trash[index].width=64;
		SpritePlane.Trash[index].height=64;
		SpritePlane.Trash[index].vx=0;
		SpritePlane.Trash[index].vy=TRASH_Y_VELOCITY-1;
		SpritePlane.Trash[index].visible=false;
	}

	//set helicopter properties
	SpriteHelicopter.width=128;
	SpriteHelicopter.height=56;
	SpriteHelicopter.x=500;
	SpriteHelicopter.y=10;
	SpriteHelicopter.vx=HELICOPTER_SPEED;
	SpriteHelicopter.alive=false;
	SpriteHelicopter.visible=false;
	SpriteHelicopter.animdelay=80;
	SpriteHelicopter.animcount=GetTickCount();
	SpriteHelicopter.framecount=0;
	SpriteHelicopter.facingright=true;
	//set trash properties
	SpriteHelicopter.TrashCount=0;
	SpriteHelicopter.TrashDelay=0;
	SpriteHelicopter.TrashTime=GetTickCount();
	for(int index=0;index<TRASH_MAXIMUM_COUNT;index++)
	{
		SpriteHelicopter.Trash[index].x=700;
		SpriteHelicopter.Trash[index].y=0;
		SpriteHelicopter.Trash[index].width=48;
		SpriteHelicopter.Trash[index].height=48;
		SpriteHelicopter.Trash[index].vx=0;
		SpriteHelicopter.Trash[index].vy=TRASH_Y_VELOCITY;
		SpriteHelicopter.Trash[index].visible=false;
	}

	//set Cessna properties
	SpriteCessna.width=120;
	SpriteCessna.height=56;
	SpriteCessna.x=700;
	SpriteCessna.y=10;
	SpriteCessna.vx=-CESSNA_SPEED;
	SpriteCessna.alive=false;
	SpriteCessna.visible=false;
	SpriteCessna.animdelay=80;
	SpriteCessna.animcount=GetTickCount();
	SpriteCessna.framecount=0;
	SpriteCessna.facingright=false;
	//set trash properties
	SpriteCessna.TrashCount=0;
	SpriteCessna.TrashDelay=0;
	SpriteCessna.TrashTime=GetTickCount();
	SpriteCessna.flyoffcount=0;
	SpriteCessna.flyoffdelay=CESSNA_FLYOFF_DELAY;
	for(int index=0;index<TRASH_MAXIMUM_COUNT;index++)
	{
		SpriteCessna.Trash[index].x=700;
		SpriteCessna.Trash[index].y=0;
		SpriteCessna.Trash[index].width=48;
		SpriteCessna.Trash[index].height=48;
		SpriteCessna.Trash[index].vx=0;
		SpriteCessna.Trash[index].vy=TRASH_Y_VELOCITY-1;
		SpriteCessna.Trash[index].visible=false;
	}

	
	//load sound into object
	sound_1=LoadSound("hit1.wav");//doh
	//if sounds don't load, fail on game init
	if(sound_1==NULL)
	{return 0;}

	//font
	// Create a D3DX font object
	D3DXCreateFont(d3ddev, 20, 0, FW_BOLD, 0, FALSE, DEFAULT_CHARSET, 
		OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, TEXT("Arial"), 
		&font_arial );
	//set font position rect
	rectFont.left=2;
	rectFont.right=200;
	rectFont.top=5;
	rectFont.bottom=rectFont.top+20;

	rectTextHealth.left=2;
	rectTextHealth.right=150;
	rectTextHealth.top=30;
	rectTextHealth.bottom=50;

	rectTextLakeHealth.left=2;
	rectTextLakeHealth.right=150;
	rectTextLakeHealth.top=55;
	rectTextLakeHealth.bottom=75;

	rectTextLevel.left=2;
	rectTextLevel.right=150;
	rectTextLevel.top=80;
	rectTextLevel.bottom=100;

	cerr<<"Game Specific Objects Initialized\n";

	//after objects initialized, go to game menu
	GameState=GAME_LEVEL_RESET;

	return 1;
}

//main game loop
void Game_Run(HWND hwnd)
{
	//set timing
	start=GetTickCount();
	//update mouse movement
	Poll_Mouse();
	//update keyboard state
	Poll_Keyboard();

	//keyboard pressed variables
	//If a loop is gone through with the key up, it hasn't been pressed yet
	if(!Key_Down(DIK_RIGHT))
	{RightKeyPressed=false;}
	if(!Key_Down(DIK_LEFT))
	{LeftKeyPressed=false;}
	if(!Key_Down(DIK_UP))
	{UpKeyPressed=false;}
	if(!Key_Down(DIK_DOWN))
	{DownKeyPressed=false;}
	if(!Key_Down(DIK_A))
	{AKeyPressed=false;}
	if(!Key_Down(DIK_D))
	{DKeyPressed=false;}
	if(!Key_Down(DIK_W))
	{WKeyPressed=false;}
	if(!Key_Down(DIK_S))
	{SKeyPressed=false;}
	if(!Key_Down(DIK_F))
	{FKeyPressed=false;}
	if(!Key_Down(DIK_RCONTROL))
	{RControlKeyPressed=false;}
	if(!Key_Down(DIK_ESCAPE))
	{EscKeyPressed=false;}
	if(!Key_Down(DIK_RETURN))
	{EnterKeyPressed=false;}
	if(!Key_Down(DIK_EQUALS))
	{PlusKeyPressed=false;}
	if(!Key_Down(DIK_MINUS))
	{MinusKeyPressed=false;}

	//main game state router
	switch(GameState)
	{
		//main menu
		case GAME_MENU:

			break;

		case GAME_LEVEL_RESET:
			//hide all enemies
			SpriteCessna.alive=false;
			SpriteCessna.visible=false;
			SpriteHelicopter.alive=false;
			SpriteHelicopter.visible=false;
			SpritePlane.alive=false;
			SpritePlane.visible=false;
			SpriteUFO.alive=false;
			SpriteUFO.visible=false;

			//hide all trash
			for(int index=0;index<TRASH_MAXIMUM_COUNT;index++)
			{
				SpriteCessna.Trash[index].visible=false;
				SpriteHelicopter.Trash[index].visible=false;
				SpritePlane.Trash[index].visible=false;
				SpriteUFO.Trash[index].visible=false;
			}
			//hide all ammo
			for(int index=0;index<AMMO_MAXIMUM_COUNT;index++)
			{
				SpriteAmmo[index].visible=false;
			}
			
			//reset enemy variables
			SpriteCessna.TrashCount=0;
			SpriteCessna.TrashTime=GetTickCount();
			//SpriteCessna.x=700;
			SpriteCessna.x=rand()%(SCREEN_WIDTH-SpriteCessna.width);
			SpriteCessna.facingright=false;
			SpriteCessna.vx=-CESSNA_SPEED;
			SpriteCessna.facingright=false;

			SpriteHelicopter.TrashCount=0;
			SpriteHelicopter.TrashTime=GetTickCount();
			//SpriteHelicopter.x=500;
			SpriteHelicopter.x=rand()%(SCREEN_WIDTH-SpriteHelicopter.width);
			SpriteHelicopter.facingright=true;
			SpriteHelicopter.vx=HELICOPTER_SPEED;

			SpritePlane.TrashCount=0;
			SpritePlane.TrashTime=GetTickCount();
			//SpritePlane.x=10;
			SpritePlane.x=rand()%(SCREEN_WIDTH-SpritePlane.width);
			SpritePlane.facingright=true;
			SpritePlane.vx=PLANE_SPEED;

			SpriteUFO.TrashCount=0;
			SpriteUFO.TrashTime=GetTickCount();
			//SpriteUFO.x=800;
			SpriteUFO.x=rand()%(SCREEN_WIDTH-SpriteUFO.width);
			SpriteUFO.facingright=false;
			SpriteUFO.vx=-UFO_SPEED;

			//reset player variables
			SpritePlayer.x=100;
			AmmoCount=0;

			//reset level variables
			LakeHealth=MAX_LAKEHEALTH;
			SpritePlayer.health=MAX_PLAYERHEALTH;
			LevelTime=GetTickCount();

			//load the new level
			GameState=GAME_LEVEL_LOAD;

			break;

		case GAME_LEVEL_LOAD:
			if(GameLevel==1)
			{
				SpriteCessna.alive=true;
				SpriteCessna.visible=true;

				LevelWinTime=30000;
				cerr<<"why"<<endl;
			}

			if(GameLevel==2)
			{
				SpriteCessna.alive=true;
				SpriteCessna.visible=true;
				SpriteHelicopter.alive=true;
				SpriteHelicopter.visible=true;

				LevelWinTime=35000;
			}

			if(GameLevel==3)
			{
				SpriteCessna.alive=true;
				SpriteCessna.visible=true;
				SpriteHelicopter.alive=true;
				SpriteHelicopter.visible=true;
				SpritePlane.alive=true;
				SpritePlane.visible=true;
				LevelWinTime=35000;
			}

			if(GameLevel==4)
			{
				SpriteCessna.alive=true;
				SpriteCessna.visible=true;
				SpriteHelicopter.alive=true;
				SpriteHelicopter.visible=true;
				SpritePlane.alive=true;
				SpritePlane.visible=true;
				LevelWinTime=35000;
			}

			if(GameLevel==5)
			{
				SpriteCessna.alive=true;
				SpriteCessna.visible=true;
				SpriteHelicopter.alive=true;
				SpriteHelicopter.visible=true;
				SpritePlane.alive=true;
				SpritePlane.visible=true;
				SpriteUFO.alive=true;
				SpriteUFO.visible=true;
				LevelWinTime=35000;
			}
			
			GameState=GAME_RUNNING;

			break;

		//game is currently running (play mode)
		case GAME_RUNNING:

			//Sound test, Removed at Release Version
			//if 'B' key is pressed, play sound
			if(Key_Down(DIK_B))
			{PlaySound(sound_1);}

			//hack for now to change the level in game
			if(Key_Down(DIK_EQUALS) && !PlusKeyPressed)
			{
				if(GameLevel<5)
					GameLevel++;
				GameState=GAME_LEVEL_RESET;

				PlusKeyPressed=true;
			}
			if(Key_Down(DIK_MINUS) && !MinusKeyPressed)
			{
				if(GameLevel>1)
					GameLevel--;
				GameState=GAME_LEVEL_RESET;

				MinusKeyPressed=true;
			}

			//see if explosions should disappear
			for(int index=0;index<EXPLOSION_MAXIMUM_COUNT;index++)
			{
				if(++SpriteExplosion[index].alivetime>=EXPLOSION_LIFE)
				{
					SpriteExplosion[index].visible=false;
				}
			}
			//player movement
			//right key, move right
			if(Key_Down(DIK_RIGHT))
			{
				SpritePlayer.x+=SpritePlayer.vx;

				SpritePlayer.facingright=true;

				//gets the current tick count to find out how many ticks have passed since last frame
				//player will only animate if it has been moving for a certain number of ticks,
				//set by animdelay
				if(GetTickCount()-SpritePlayer.animcount>=SpritePlayer.animdelay)
				{
					//if enough ticks have passed, change frame of animation
					SpritePlayer.framecount++;
					if(SpritePlayer.framecount>=PLAYER_FRAMES_OF_ANIMATION)
						SpritePlayer.framecount=0;
					
					//reset animation count information (this will be reset to current tick count later)
					SpritePlayer.animcount=0;
				}
				//if animation already happened, reset counter to current tick count
				if(SpritePlayer.animcount==0)
				{
					SpritePlayer.animcount=GetTickCount();
				}
			}
			//if left key, move left
			if(Key_Down(DIK_LEFT))
			{
				SpritePlayer.x-=SpritePlayer.vx;

				SpritePlayer.facingright=false;

				if(GetTickCount()-SpritePlayer.animcount>=SpritePlayer.animdelay)
				{
					//if enough ticks have passed, change frame of animation
					SpritePlayer.framecount++;
					if(SpritePlayer.framecount>=PLAYER_FRAMES_OF_ANIMATION)
						SpritePlayer.framecount=0;
					
					//reset animation count information (this will be reset to current tick count later)
					SpritePlayer.animcount=0;
				}
				//if animation already happened, reset counter to current tick count
				if(SpritePlayer.animcount==0)
				{
					SpritePlayer.animcount=GetTickCount();
				}
			}

			//move UFO if it's alive
			if(SpriteUFO.alive)
			{
				SpriteUFO.x+=SpriteUFO.vx;
			
				if(GetTickCount()-SpriteUFO.animcount>=SpriteUFO.animdelay)
				{
					SpriteUFO.framecount++;
					if(SpriteUFO.framecount>=UFO_FRAMES_OF_ANIMATION)
						SpriteUFO.framecount=0;
					
					//reset animation count information (this will be reset to current tick count later)
					SpriteUFO.animcount=0;
				}
				//if animation already happened, reset counter to current tick count
				if(SpriteUFO.animcount==0)
				{
					SpriteUFO.animcount=GetTickCount();
				}
			}
			//move plane if it's alive
			if(SpritePlane.alive)
			{
				SpritePlane.x+=SpritePlane.vx;
			
				if(GetTickCount()-SpritePlane.animcount>=SpritePlane.animdelay)
				{
					SpritePlane.framecount++;
					if(SpritePlane.framecount>=PLANE_FRAMES_OF_ANIMATION)
						SpritePlane.framecount=0;
					
					//reset animation count information (this will be reset to current tick count later)
					SpritePlane.animcount=0;
				}
				//if animation already happened, reset counter to current tick count
				if(SpritePlane.animcount==0)
				{
					SpritePlane.animcount=GetTickCount();
				}
			}
			//move helicopter if it's alive
			if(SpriteHelicopter.alive)
			{
				SpriteHelicopter.x+=SpriteHelicopter.vx;
			
				if(GetTickCount()-SpriteHelicopter.animcount>=SpriteHelicopter.animdelay)
				{
					SpriteHelicopter.framecount++;
					if(SpriteHelicopter.framecount>=HELICOPTER_FRAMES_OF_ANIMATION)
						SpriteHelicopter.framecount=0;
					
					//reset animation count information (this will be reset to current tick count later)
					SpriteHelicopter.animcount=0;
				}
				//if animation already happened, reset counter to current tick count
				if(SpriteHelicopter.animcount==0)
				{
					SpriteHelicopter.animcount=GetTickCount();
				}
			}
			//move cessna if it's alive
			if(SpriteCessna.alive && SpriteCessna.visible)
			{
				SpriteCessna.x+=SpriteCessna.vx;

				if(GetTickCount()-SpriteCessna.animcount>=SpriteCessna.animdelay)
				{
					SpriteCessna.framecount++;
					if(SpriteCessna.framecount>=CESSNA_FRAMES_OF_ANIMATION)
						SpriteCessna.framecount=0;
					
					//reset animation count information (this will be reset to current tick count later)
					SpriteCessna.animcount=0;
				}
				//if animation already happened, reset counter to current tick count
				if(SpriteCessna.animcount==0)
				{
					SpriteCessna.animcount=GetTickCount();
				}

			}

			//if the sprite is alive but not visible (offscreen)
			//cessna
			if(SpriteCessna.alive && !SpriteCessna.visible)
			{
				//increment flyoffcount, see if it's greater than or equal to the flyoff delay
				if(++SpriteCessna.flyoffcount >= SpriteCessna.flyoffdelay)
				{
					//if it is, spawn the cessna randomly on the left or right so it flies through the screen
					//and make it visible again
					if(rand()%2==0)
					{
						SpriteCessna.x=0-SpriteCessna.width;
						SpriteCessna.vx=CESSNA_SPEED;
						SpriteCessna.facingright=true;
						SpriteCessna.visible=true;
						SpriteCessna.flyoffcount=0;
					}
					else
					{
						SpriteCessna.x=SCREEN_WIDTH;
						SpriteCessna.vx=-CESSNA_SPEED;
						SpriteCessna.facingright=false;
						SpriteCessna.visible=true;
						SpriteCessna.flyoffcount=0;
					}
				}
			}
			//plane
			if(SpritePlane.alive && !SpritePlane.visible)
			{
				//increment flyoffcount, see if it's greater than or equal to the flyoff delay
				if(++SpritePlane.flyoffcount >= SpritePlane.flyoffdelay)
				{
					//if it is, spawn the plane randomly on the left or right so it flies through the screen
					//and make it visible again
					if(rand()%2==0)
					{
						SpritePlane.x=0-SpritePlane.width;
						SpritePlane.vx=PLANE_SPEED;
						SpritePlane.facingright=true;
						SpritePlane.visible=true;
						SpritePlane.flyoffcount=0;
					}
					else
					{
						SpritePlane.x=SCREEN_WIDTH;
						SpritePlane.vx=-PLANE_SPEED;
						SpritePlane.facingright=false;
						SpritePlane.visible=true;
						SpritePlane.flyoffcount=0;
					}
				}
			}


			//Fire Ammo
			if((Key_Down(DIK_F) && !FKeyPressed) || (Key_Down(DIK_RCONTROL) && !RControlKeyPressed))
			{
				FKeyPressed=true;
				RControlKeyPressed=true;
				//set ammo to player's position (middle of player)
				SpriteAmmo[AmmoCount].x=SpritePlayer.x+(SpritePlayer.width/2)-(SpriteAmmo[AmmoCount].width/2);
				SpriteAmmo[AmmoCount].y=SpritePlayer.y;
				//reset to initial velocity
				SpriteAmmo[AmmoCount].vy=-AMMO_VELOCITY;
				if(AMMO_X_MOTION)
				{
					SpriteAmmo[AmmoCount].vx=1;	
				}
				//show ammo
				SpriteAmmo[AmmoCount].visible=true;
				//Advance array to next piece of ammo
				AmmoCount++;
				//if all the ammo has been fired, restart with first piece
				if(AmmoCount==AMMO_MAXIMUM_COUNT-1)
				{
					AmmoCount=0;
				}
			}
			//move ammo
			//change velocity based on gravity and move ammo
			for(int index=0;index<AMMO_MAXIMUM_COUNT;index++)
			{
				SpriteAmmo[index].vy+=GRAVITY_ACCELERATION;
				SpriteAmmo[index].y+=SpriteAmmo[index].vy;
				SpriteAmmo[index].x+=SpriteAmmo[index].vx;
			}

			//fire trash at random intervals (probably will change), only if sprite is alive
			//UFO's Trash
			//if the delay between the last fire has passed
			if( SpriteUFO.alive && (GetTickCount()-SpriteUFO.TrashTime) >= SpriteUFO.TrashDelay)
			{
				FireTrash(&SpriteUFO,TRASH_FUNCTION_UFO);
				//get a new random spawn time
				SpriteUFO.TrashDelay=rand()%UFO_TRASH_DELAY+1000;
			}
			//Plane's trash (plane must be visible)
			if( SpritePlane.alive && SpritePlane.visible &&
				(GetTickCount()-SpritePlane.TrashTime) >= SpritePlane.TrashDelay)
			{
				//if the drop point is offscreen, hold off on firing trash till it's in view
				if( (SpritePlane.facingright && SpritePlane.x < 0 ) ||
					(!SpritePlane.facingright && SpritePlane.x+SpritePlane.width > SCREEN_WIDTH) )
				{}
				else
				{
					FireTrash(&SpritePlane,TRASH_FUNCTION_PLANE);
					//get a new random spawn time
					SpritePlane.TrashDelay=rand()%PLANE_TRASH_DELAY+1000;
				}
			}
			//helicopter's trash
			if( SpriteHelicopter.alive && (GetTickCount()-SpriteHelicopter.TrashTime) >= SpriteHelicopter.TrashDelay)
			{
				FireTrash(&SpriteHelicopter,TRASH_FUNCTION_HELICOPTER);
				//get a new random spawn time
				SpriteHelicopter.TrashDelay=rand()%HELICOPTER_TRASH_DELAY+1000;
			}
			//cessna's trash (cessna must be visible)
			if( SpriteCessna.alive && SpriteCessna.visible &&
				(GetTickCount()-SpriteCessna.TrashTime) >= SpriteCessna.TrashDelay)
			{
				//if the drop point is offscreen, hold off on firing trash till it's in view
				if( (SpriteCessna.facingright && SpriteCessna.x < 0 ) ||
					(!SpriteCessna.facingright && SpriteCessna.x+SpriteCessna.width > SCREEN_WIDTH) )
				{}
				else
				{
					FireTrash(&SpriteCessna,TRASH_FUNCTION_CESSNA);
					//get a new random spawn time
					SpriteCessna.TrashDelay=rand()%CESSNA_TRASH_DELAY+1000;
				}
			}

			//move trash
			//we don't worry if the trash is visible or not, it's not cpu intensive
			for(int index=0;index<TRASH_MAXIMUM_COUNT;index++)
			{
				//ufo trash
				SpriteUFO.Trash[index].y+=SpriteUFO.Trash[index].vy;
				SpriteUFO.Trash[index].x+=SpriteUFO.Trash[index].vx;
				//plane trash
				SpritePlane.Trash[index].y+=SpritePlane.Trash[index].vy;
				SpritePlane.Trash[index].x+=SpritePlane.Trash[index].vx;
				//helicopter trash
				SpriteHelicopter.Trash[index].y+=SpriteHelicopter.Trash[index].vy;
				SpriteHelicopter.Trash[index].x+=SpriteHelicopter.Trash[index].vx;
				//cessna trash
				SpriteCessna.Trash[index].y+=SpriteCessna.Trash[index].vy;
				SpriteCessna.Trash[index].x+=SpriteCessna.Trash[index].vx;
			}

			//Collision Detection
			//Player
			//with walls
			CollisionWithWalls(&SpritePlayer);
			//UFO with walls, if alive
			if( SpriteUFO.alive && (SpriteUFO.x<=0 || SpriteUFO.x+SpriteUFO.width >= SCREEN_WIDTH))
			{
				SpriteUFO.vx=-SpriteUFO.vx;
			}
			//helicopter with walls
			if( SpriteHelicopter.alive && (SpriteHelicopter.x<=0 || SpriteHelicopter.x+SpriteHelicopter.width >= SCREEN_WIDTH))
			{
				SpriteHelicopter.vx=-SpriteHelicopter.vx;
				//reverse facing direction
				SpriteHelicopter.facingright=!SpriteHelicopter.facingright;
			}
			//Cessna with walls
			if( SpriteCessna.alive && (SpriteCessna.x+SpriteCessna.width<0 || 
				SpriteCessna.x > SCREEN_WIDTH) && SpriteCessna.flyoffcount==0)
			{
				//SpriteCessna.vx=-SpriteCessna.vx;
				SpriteCessna.flyoffcount=1;
				SpriteCessna.visible=false;
			}
			//plane with walls
			if( SpritePlane.alive && (SpritePlane.x+SpritePlane.width < 0 || 
				SpritePlane.x > SCREEN_WIDTH) && SpritePlane.flyoffcount==0)
			{
				SpritePlane.flyoffcount=1;
				SpritePlane.visible=false;
			}

			//Ammo with Trash
			//for every ammo
			for(int index=0;index<AMMO_MAXIMUM_COUNT;index++)
			{
				//if it's visible, check collisions
				if(SpriteAmmo[index].visible==true)
				{
					//check every ammo against every trash
					for(int index2=0;index2<TRASH_MAXIMUM_COUNT;index2++)
					{
						//but only do the check if the trash is visible
						//ufo trash
						if(SpriteUFO.Trash[index2].visible==true)
						{
							//if there is a collision
							if(CollisionAmmoWithTrash(&SpriteAmmo[index],&SpriteUFO.Trash[index2]))
							{
								//hide both the trash and the ammo
								SpriteAmmo[index].visible=false;
								SpriteUFO.Trash[index2].visible=false;
							}
						}
						//plane trash
						if(SpritePlane.Trash[index2].visible==true)
						{
							//if there is a collision
							if(CollisionAmmoWithTrash(&SpriteAmmo[index],&SpritePlane.Trash[index2]))
							{
								//hide both the trash and the ammo
								SpriteAmmo[index].visible=false;
								SpritePlane.Trash[index2].visible=false;
							}
						}
						//helicopter trash
						if(SpriteHelicopter.Trash[index2].visible==true)
						{
							//if there is a collision
							if(CollisionAmmoWithTrash(&SpriteAmmo[index],&SpriteHelicopter.Trash[index2]))
							{
								//hide both the trash and the ammo
								SpriteAmmo[index].visible=false;
								SpriteHelicopter.Trash[index2].visible=false;
							}
						}
						//Cessna trash
						if(SpriteCessna.Trash[index2].visible==true)
						{
							//if there is a collision
							if(CollisionAmmoWithTrash(&SpriteAmmo[index],&SpriteCessna.Trash[index2]))
							{
								//hide both the trash and the ammo
								SpriteAmmo[index].visible=false;
								SpriteCessna.Trash[index2].visible=false;
							}
						}
					}
				}
			}//end ammo against trash check
			
			//ammo with ground
			for(int index=0;index<AMMO_MAXIMUM_COUNT;index++)
			{
				//if the ammo is visible
				if(SpriteAmmo[index].visible==true)
				{
					//and it hits the ground
					if(CollisionObjectWithFloor(SpriteAmmo[index].y))
					{
						//make it invisible
						SpriteAmmo[index].visible=false;
						//have an explosion set to it's x position minus half the explosion's width
						SpriteExplosion[ExplosionCount].x = (SpriteAmmo[index].x+SpriteAmmo[index].width/2) -
							(SpriteExplosion[ExplosionCount].width/2);
						//make the explosion visible
						SpriteExplosion[ExplosionCount].visible=true;
						//set the explosions life
						SpriteExplosion[ExplosionCount].alivetime=1;
						//fire the next explosion in the array
						ExplosionCount++;
						//if we're at the final explosion, start over
						if(ExplosionCount>=EXPLOSION_MAXIMUM_COUNT)
						{
							ExplosionCount=0;
						}

						//subtract health from the player equal to the damage he should take
						//player position
						SpritePlayer.health-=DamageFromCollision(SpriteAmmo[index].x+(SpriteAmmo[index].width/2),
							SpritePlayer.x+(SpritePlayer.width/2),SpriteExplosion[0].width);
					}
				}
			}//end ammo with ground checks

			//trash with ground and side walls
			for(int index=0;index<TRASH_MAXIMUM_COUNT;index++)
			{
				//UFO Trash
				if(SpriteUFO.Trash[index].visible==true)
				{
					//side walls ( so they don't fly off the screen)
					if(SpriteUFO.Trash[index].x<=0)
					{
						SpriteUFO.Trash[index].x=0;
						SpriteUFO.Trash[index].vx=0;
					}
					else if(SpriteUFO.Trash[index].x+SpriteUFO.Trash[index].width>=SCREEN_WIDTH)
					{
						SpriteUFO.Trash[index].x=SCREEN_WIDTH-SpriteUFO.Trash[index].width;
						SpriteUFO.Trash[index].vx=0;
					}

					//floor
					if(CollisionObjectWithFloor(SpriteUFO.Trash[index].y+SpriteUFO.Trash[index].height))
					{
						SpriteUFO.Trash[index].visible=false;
						LakeHealth--;
					}
				}
				//Plane Trash
				if(SpritePlane.Trash[index].visible==true)
				{
					//side walls ( so they don't fly off the screen)
					if(SpritePlane.Trash[index].x<=0)
					{
						SpritePlane.Trash[index].x=0;
						SpritePlane.Trash[index].vx=0;
					}
					else if(SpritePlane.Trash[index].x+SpritePlane.Trash[index].width>=SCREEN_WIDTH)
					{
						SpritePlane.Trash[index].x=SCREEN_WIDTH-SpritePlane.Trash[index].width;
						SpritePlane.Trash[index].vx=0;
					}
					//floor
					if(CollisionObjectWithFloor(SpritePlane.Trash[index].y+SpritePlane.Trash[index].height))
					{
						SpritePlane.Trash[index].visible=false;
						LakeHealth--;
					}
				}
				//helicopter trash
				if(SpriteHelicopter.Trash[index].visible==true)
				{
					//side walls ( so they don't fly off the screen)
					if(SpriteHelicopter.Trash[index].x<=0)
					{
						SpriteHelicopter.Trash[index].x=0;
						SpriteHelicopter.Trash[index].vx=0;
					}
					else if(SpriteHelicopter.Trash[index].x+SpriteHelicopter.Trash[index].width>=SCREEN_WIDTH)
					{
						SpriteHelicopter.Trash[index].x=SCREEN_WIDTH-SpriteHelicopter.Trash[index].width;
						SpriteHelicopter.Trash[index].vx=0;
					}
					//floor
					if(CollisionObjectWithFloor(SpriteHelicopter.Trash[index].y+SpriteHelicopter.Trash[index].height))
					{
						SpriteHelicopter.Trash[index].visible=false;
						LakeHealth--;
					}
				}
				//Cessna trash
				if(SpriteCessna.Trash[index].visible==true)
				{
					//side walls ( so they don't fly off the screen)
					if(SpriteCessna.Trash[index].x<=0)
					{
						SpriteCessna.Trash[index].x=0;
						SpriteCessna.Trash[index].vx=0;
					}
					else if(SpriteCessna.Trash[index].x+SpriteCessna.Trash[index].width>=SCREEN_WIDTH)
					{
						SpriteCessna.Trash[index].x=SCREEN_WIDTH-SpriteCessna.Trash[index].width;
						SpriteCessna.Trash[index].vx=0;
					}
					//floor
					if(CollisionObjectWithFloor(SpriteCessna.Trash[index].y+SpriteCessna.Trash[index].height))
					{
						SpriteCessna.Trash[index].visible=false;
						LakeHealth--;
					}
				}
			}

			//check for level victory
			if(GetTickCount()-LevelTime>=LevelWinTime)
			{
				GameLevel++;
				GameState=GAME_LEVEL_RESET;
				cerr<<"tick "<<GetTickCount()<<" leveltime "<<LevelTime<<" wintime "<<LevelWinTime<<endl;
			}
			
			break;

		default:
			//this option should never come up, if it does, there are problems
			cerr<<"Problem with Game State Switch\n";
			break;
	}

	//rects for blitting onto backbuffer
	//player rects
	//left of rect = 1(border)+framecount(borders of all the frames)+number of frames (determined by width*framecount)
	SpritePlayer.rectsource.left = (SpritePlayer.framecount*SpritePlayer.width)+1+SpritePlayer.framecount;
	SpritePlayer.rectsource.right = SpritePlayer.rectsource.left+SpritePlayer.width;
	if(SpritePlayer.facingright)
	{
		SpritePlayer.rectsource.top=1;
		SpritePlayer.rectsource.bottom=SpritePlayer.rectsource.top+SpritePlayer.height;
	}
	else
	{
		SpritePlayer.rectsource.top=SpritePlayer.height+2;
		SpritePlayer.rectsource.bottom=SpritePlayer.rectsource.top+SpritePlayer.height;
	}

	//enemy rects
	//ufo
	SpriteUFO.rectsource.left = (SpriteUFO.framecount*SpriteUFO.width)+1+SpriteUFO.framecount;
	SpriteUFO.rectsource.right= SpriteUFO.rectsource.left+SpriteUFO.width;
	SpriteUFO.rectsource.top=1;
	SpriteUFO.rectsource.bottom=SpriteUFO.rectsource.top+SpriteUFO.height;
	//helicopter
	SpriteHelicopter.rectsource.left = (SpriteHelicopter.framecount*SpriteHelicopter.width)+1+SpriteHelicopter.framecount;
	SpriteHelicopter.rectsource.right= SpriteHelicopter.rectsource.left+SpriteHelicopter.width;
	if(SpriteHelicopter.facingright)
	{
		SpriteHelicopter.rectsource.top=SpriteHelicopter.height+2;
		SpriteHelicopter.rectsource.bottom=SpriteHelicopter.rectsource.top+SpriteHelicopter.height;
	}
	else
	{
		SpriteHelicopter.rectsource.top=1;
		SpriteHelicopter.rectsource.bottom=SpriteHelicopter.rectsource.top+SpriteHelicopter.height;
	}
	//plane
	SpritePlane.rectsource.left = (SpritePlane.framecount*SpritePlane.width)+1+SpritePlane.framecount;
	SpritePlane.rectsource.right= SpritePlane.rectsource.left+SpritePlane.width;
	if(SpritePlane.facingright)
	{
		SpritePlane.rectsource.top=SpritePlane.height+2;
		SpritePlane.rectsource.bottom=SpritePlane.rectsource.top+SpritePlane.height;
	}
	else
	{
		SpritePlane.rectsource.top=1;
		SpritePlane.rectsource.bottom=SpritePlane.rectsource.top+SpritePlane.height;
	}
	//cessna
	SpriteCessna.rectsource.left = (SpriteCessna.framecount*SpriteCessna.width)+1+SpriteCessna.framecount;
	SpriteCessna.rectsource.right= SpriteCessna.rectsource.left+SpriteCessna.width;
	if(SpriteCessna.facingright)
	{
		SpriteCessna.rectsource.top=SpriteCessna.height+2;
		SpriteCessna.rectsource.bottom=SpriteCessna.rectsource.top+SpriteCessna.height;
	}
	else
	{
		SpriteCessna.rectsource.top=1;
		SpriteCessna.rectsource.bottom=SpriteCessna.rectsource.top+SpriteCessna.height;
	}

	//begin rendering scene
	if(SUCCEEDED(d3ddev->BeginScene()))
	{
		//clear back buffer
		d3ddev->Clear(0,NULL,D3DCLEAR_TARGET,D3DCOLOR_XRGB(0,0,255),1.0f,0);

		//draw background to backbuffer
		d3ddev->StretchRect(background_surface,NULL,backbuffer,NULL,D3DTEXF_NONE);

		//begin sprite drawing
		sprite_handler->Begin(D3DXSPRITE_ALPHABLEND);

		//position is a vector that stores sprite position in 3d coordinates
		//these if cases check to see if the sprite should be displayed, then draws it using
		//the spriter handler, with transparency

		//draw explosions
		for(int index=0;index<EXPLOSION_MAXIMUM_COUNT;index++)
		{
			if(SpriteExplosion[index].visible==true)
			{
				position.x=(float)SpriteExplosion[index].x;
				position.y=(float)SpriteExplosion[index].y;
				sprite_handler->Draw(SpriteExplosion[0].texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
			}
		}
		//Draw Player
		if(SpritePlayer.visible==true)
		{
			//vector holding sprite position (for sprite handler)
			position.x=(float)SpritePlayer.x;
			position.y=(float)SpritePlayer.y;
			//render sprite
			sprite_handler->Draw(SpritePlayer.texture,&SpritePlayer.rectsource,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}
		//Draw UFO
		if(SpriteUFO.visible==true)
		{
			position.x=(float)SpriteUFO.x;
			position.y=(float)SpriteUFO.y;
			sprite_handler->Draw(SpriteUFO.texture,&SpriteUFO.rectsource,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}
		//Draw Plane
		if(SpritePlane.visible==true)
		{
			position.x=(float)SpritePlane.x;
			position.y=(float)SpritePlane.y;
			sprite_handler->Draw(SpritePlane.texture,&SpritePlane.rectsource,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}
		//draw helicopter
		if(SpriteHelicopter.visible==true)
		{
			position.x=(float)SpriteHelicopter.x;
			position.y=(float)SpriteHelicopter.y;
			sprite_handler->Draw(SpriteHelicopter.texture,&SpriteHelicopter.rectsource,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}
		//draw Cessna
		if(SpriteCessna.visible==true)
		{
			position.x=(float)SpriteCessna.x;
			position.y=(float)SpriteCessna.y;
			sprite_handler->Draw(SpriteCessna.texture,&SpriteCessna.rectsource,NULL,&position,D3DCOLOR_XRGB(255,255,255));
		}
		//Draw all Ammo
		for(int index=0;index<AMMO_MAXIMUM_COUNT;index++)
		{
			if(SpriteAmmo[index].visible==true)
			{
				position.x=(float)SpriteAmmo[index].x;
				position.y=(float)SpriteAmmo[index].y;
				//use the texture from the first in array
				sprite_handler->Draw(SpriteAmmo[0].texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
			}
		}
		//draw trash
		for(int index=0;index<TRASH_MAXIMUM_COUNT;index++)
		{
			//ufo trash
			if(SpriteUFO.Trash[index].visible==true)
			{
				position.x=(float)SpriteUFO.Trash[index].x;
				position.y=(float)SpriteUFO.Trash[index].y;
				sprite_handler->Draw(SpriteUFO.Trash[0].texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
			}
			//plane trash
			if(SpritePlane.Trash[index].visible==true)
			{
				position.x=(float)SpritePlane.Trash[index].x;
				position.y=(float)SpritePlane.Trash[index].y;
				sprite_handler->Draw(SpritePlane.Trash[0].texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
			}
			//helicopter trash
			if(SpriteHelicopter.Trash[index].visible==true)
			{
				position.x=(float)SpriteHelicopter.Trash[index].x;
				position.y=(float)SpriteHelicopter.Trash[index].y;
				sprite_handler->Draw(SpriteHelicopter.Trash[0].texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
			}
			//Cessna trash
			if(SpriteCessna.Trash[index].visible==true)
			{
				position.x=(float)SpriteCessna.Trash[index].x;
				position.y=(float)SpriteCessna.Trash[index].y;
				sprite_handler->Draw(SpriteCessna.Trash[0].texture,NULL,NULL,&position,D3DCOLOR_XRGB(255,255,255));
			}
		}
		
		//Draw Text
		//draw the current FPS, and player's health in the upper left corner
		float FramesPerSecond=(float)GetTickCount()-start+15;
		FramesPerSecond=1000/FramesPerSecond;
		char StringFPS[100];
		char StringHealth[50];
		char StringLakeHealth[50];
		string StringLevel="Level: ";
		stringstream ssLevel;
		ssLevel<<StringLevel<<" "<<GameLevel;
		StringLevel=ssLevel.str();

		sprintf(StringFPS,"FPS: %f",FramesPerSecond);
		sprintf(StringHealth,"Health: %d",SpritePlayer.health);
		sprintf(StringLakeHealth,"Lake Health: %d",LakeHealth);
		font_arial->DrawText(sprite_handler, StringFPS, -1, &rectFont,0, D3DCOLOR_XRGB(0,0,0));
		font_arial->DrawText(sprite_handler,StringHealth,-1,&rectTextHealth,0,D3DCOLOR_XRGB(0,0,0));
		font_arial->DrawText(sprite_handler,StringLakeHealth,-1,&rectTextLakeHealth,0,D3DCOLOR_XRGB(0,0,0));
		font_arial->DrawText(sprite_handler,StringLevel.c_str(),-1,&rectTextLevel,0,D3DCOLOR_XRGB(0,0,0));

		//end sprite drawing
		sprite_handler->End();

		d3ddev->EndScene();
	}//stop rendering scene

	//present backbuffer onto primary
	d3ddev->Present(NULL,NULL,NULL,NULL);

	//check for escape key to quit
	if(Key_Down(DIK_ESCAPE) && !EscKeyPressed)
	{
		EscKeyPressed=true;
		//quit game
		PostMessage(hwnd,WM_DESTROY,0,0);
	}
		
	//synchronize frames to 15 ms
	while(GetTickCount()-start<15)
	{}

}

void Game_End (HWND hwnd)
{
	//Release Surfaces
	background_surface->Release();
	cerr<<"Surfaces Released\n";

	//Release Textures
	SpritePlayer.texture->Release();
	SpriteAmmo[0].texture->Release();//only release first one in array (only loaded once)
	SpriteExplosion[0].texture->Release();
	SpriteUFO.texture->Release();
	SpriteUFO.Trash[0].texture->Release();
	SpritePlane.texture->Release();
	SpritePlane.Trash[0].texture->Release();
	SpriteHelicopter.texture->Release();
	SpriteHelicopter.Trash[0].texture->Release();
	SpriteCessna.texture->Release();
	SpriteCessna.Trash[0].texture->Release();
	cerr<<"Textures Released\n";

	//Release Fonts
	font_arial->Release();
	cerr<<"Fonts Released\n";

	//Delete Sounds
	if(sound_1 !=NULL)
	{delete sound_1;}
	cerr<<"Sounds Deleted\n";

	cerr<<"\nGame shut down!\n";

}


//ADDITIONAL FUNCTIONS/////////////////////////////

//collision detection against walls
void CollisionWithWalls(SPRITE_PLAYER *sprite)
{
	//if sprite hits left wall
	if(sprite->x<=0)
	{
		//set sprite position to left wall
		sprite->x=0;
	}
	//right wall
	if(sprite->x+sprite->width>=SCREEN_WIDTH)
	{
		sprite->x=SCREEN_WIDTH-sprite->width;
	}
	//top wall
	if(sprite->y<=0)
	{
		sprite->y=0;
	}
	//bottom wall
	if(sprite->y+sprite->height>=SCREEN_HEIGHT)
	{
		sprite->y=SCREEN_HEIGHT-sprite->height;
	}
}//end CollisionWithWalls

//collison between ammo and trash
bool CollisionAmmoWithTrash(SPRITE_AMMO *ammo, SPRITE_TRASH *trash)
{
	//creates rectangles of positions, plus a dummy rect for god knows what reason
	RECT rAmmo,rTrash,rDummy;

	//set rect's based on position
	rAmmo.left=ammo->x;
	rAmmo.right=ammo->x+ammo->width;
	rAmmo.top=ammo->y;
	rAmmo.bottom=ammo->y+ammo->height;

	rTrash.left=trash->x;
	rTrash.right=trash->x+trash->width;
	rTrash.top=trash->y;
	rTrash.bottom=trash->y+trash->height;

	//if they intersect at all, collision exists
	if(IntersectRect(&rDummy,&rAmmo,&rTrash))
	{return true;}

	return false;

}//end CollisionAmmoWithTrash

//collision between trash and the ground
bool CollisionObjectWithFloor(int y)
{
	if(y >=SCREEN_HEIGHT-30)
		return true;
	return false;
}

//takes the ammo's x position and the player's x position and calculates the damage that should be taken
int DamageFromCollision(int AmmoX,int PlayerX,int ExplosionRadius)
{
	//the difference in the ammo and player's position
	int DifferenceX=abs(AmmoX-PlayerX);

	//if the player is within explosion range
	if (DifferenceX<ExplosionRadius)
	{
		//take the explosion range and subtract the difference to get damage
		//ie radius is 50 and player is 10 feet away
		//player takes 40 damage
		//if player is 30 feet away, player takes 20 damage
		return ExplosionRadius-DifferenceX;
	}
	else
	{
		return 0;
	}
}

//fires the trash from the trash dropper
void FireTrash(SPRITE_ENEMY *enemy,int TrashFunctionID)
{
	//put the trash at the correct location in relation to the enemy
	switch (TrashFunctionID)
	{
	case TRASH_FUNCTION_UFO:
		//drop from middle
		enemy->Trash[enemy->TrashCount].x=enemy->x+(enemy->width/2);
		enemy->Trash[enemy->TrashCount].y=enemy->height;
		break;
	case TRASH_FUNCTION_PLANE:
		//drop from trailing end
		if(enemy->facingright)
			enemy->Trash[enemy->TrashCount].x = enemy->x - enemy->Trash[enemy->TrashCount].width;
		else
			enemy->Trash[enemy->TrashCount].x = enemy->x + enemy->width;

		enemy->Trash[enemy->TrashCount].y=enemy->height;
		break;
	case TRASH_FUNCTION_HELICOPTER:
		//drop from middle
		enemy->Trash[enemy->TrashCount].x=enemy->x+(enemy->width/2);
		enemy->Trash[enemy->TrashCount].y=enemy->height;
		break;
	case TRASH_FUNCTION_CESSNA:
		//drop from trailing end
		if(enemy->facingright)
			enemy->Trash[enemy->TrashCount].x = enemy->x - enemy->Trash[enemy->TrashCount].width;
		else
			enemy->Trash[enemy->TrashCount].x = enemy->x + enemy->width;

		enemy->Trash[enemy->TrashCount].y=enemy->height;
		break;
	default:
		cerr<<"Incorrect trash firing call\n";
	}

	enemy->Trash[enemy->TrashCount].visible=true;
	//set trash x velocity to a fraction of the dropper's velocity
	enemy->Trash[enemy->TrashCount].vx=enemy->vx*.2f;

	//let the next piece of trash be fired
	enemy->TrashCount++;
	//if we're at the end of the array, restart from the beginning
	if(enemy->TrashCount==TRASH_MAXIMUM_COUNT-1)
	{
		enemy->TrashCount=0;
	}
	//reset the trash firing delay
	enemy->TrashTime=GetTickCount();
}
