//Game Header
//All game specific code goes here
//Created 3/26/06 by Adam Sepanski

#ifndef _GAME_H
#define _GAME_H

//includes
//DirectX headers
#include <d3d9.h>
#include <d3dx9.h>
#include <dxerr9.h>
#include <dsound.h>
#include <dinput.h>
#include <windows.h>
//Regular headers
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <sstream>
#include <iostream>
//Created headers
#include "DXGraphics.H"
#include "DXAudio.h"
#include "DXInput.h"
#include "DXFont.h"

//application title
#define APPTITLE "Ted the Bastard Sloth"

const int SCREEN_WIDTH=1024;
const int SCREEN_HEIGHT=768;

const int TRASH_MAXIMUM_COUNT=15;//maximum amount of trash

//Take Keyboard Input
#define KEY_DOWN(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 1 : 0)
#define KEY_UP(vk_code)((GetAsyncKeyState(vk_code) & 0x8000) ? 1 : 0)

//player sprite structure
typedef struct
{
	float x,y;//position
	int width,height;//size
	int vx,vy;//velocity
	unsigned long int animdelay,animcount;//how long between frames
	int framecount;
	bool facingright;
	LPDIRECT3DTEXTURE9 texture;//texture to store sprite image
	RECT rectsource;//stores rectangle of frame for copying onto main surface
	bool visible;
	int health;
}SPRITE_PLAYER;

//ammunition sprite structure
typedef struct
{
	float x,y;
	int width,height;
	float vx,vy;
	unsigned long int animdelay,animcount;
	LPDIRECT3DTEXTURE9 texture;
	RECT rectsource;
	bool visible;
}SPRITE_AMMO;

typedef struct
{
	float x,y;
	int width,height;
	float vx,vy;
	unsigned long int animdelay,animcount;
	LPDIRECT3DTEXTURE9 texture;
	RECT rectsource;
	bool visible;
	int alivetime;
}SPRITE_MISC;

typedef struct
{
	float x,y;
	int width,height;
	float vx,vy;
	unsigned long int animdelay,animcount;
	LPDIRECT3DTEXTURE9 texture;
	RECT rectsource;
	bool visible;
	int alivetime;
}SPRITE_TRASH;

//enemy sprite structure
typedef struct
{
	float x,y;
	int width,height;
	float vx,vy;
	unsigned long int animdelay,animcount;//how long between frames
	int framecount;
	bool facingright;
	LPDIRECT3DTEXTURE9 texture;
	RECT rectsource;
	//difference between alive and visible, sprite may be off screen (not visible) but still alive.
	bool visible;//is the sprite visible
	bool alive;//is the sprite alive at all
	int flyoffcount;//how long the sprite has been offscreen
	int flyoffdelay;//how long the sprite should stay offscreen before respawning
	SPRITE_TRASH Trash[15];
	int TrashCount;//what trash is ready to fire
	unsigned long int TrashDelay;//random time it needs to take for trash to drop
	unsigned long int TrashTime;//current time delay for trash to drop
}SPRITE_ENEMY;

//function prototypes
int Game_Init(HWND);
void Game_Run(HWND);
void Game_End(HWND);

void CollisionWithWalls(SPRITE_PLAYER *sprite);
bool CollisionAmmoWithTrash(SPRITE_AMMO *ammo, SPRITE_TRASH *trash);
bool CollisionObjectWithFloor(int y);
int DamageFromCollision(int AmmoX,int PlayerX,int ExplosionRadius);
void FireTrash(SPRITE_ENEMY *enemy,int TrashFunctionID);

#endif