//Windows code for gaming framework
//created by Adam Sepanski on 3/26/06

//header files
//DirectX Headers
#include <d3d9.h>
#include <d3dx9.h>
//Regular Headers
#include <time.h>
#include <stdio.h>
#include <iostream>
#include <fstream>
//Created Headers
#include "DXGraphics.h"
#include "DXAudio.h"
#include "DXInput.h"
#include "Game.h"

//namespaces
using namespace std;

//Function Prototypes
//Windows Functions
LRESULT WINAPI WinProc(HWND,UINT,WPARAM,LPARAM);
ATOM MyRegisterClass(HINSTANCE);
//Game Functions
int Game_Init(HWND);
void Game_Run(HWND);
void Game_End(HWND);

//windows message handler
LRESULT WINAPI WinProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch( msg )
    {
        case WM_DESTROY:
			//all framework related objects are released here
			//game-specific objects are released in Game_Shutdown();
			//this is to keep repeating code out of the main game source

			//release d3d device
			if(d3ddev!=NULL)
			{d3ddev->Release();
			cerr<<"Direct3D Device Released\n";}
			//release d3d object
			if(d3d!=NULL)
			{d3d->Release();
			cerr<<"Direct3D Object Released\n";}

			//release sprite handler
			if(sprite_handler!=NULL)
			{
				sprite_handler->Release();
				cerr<<"Sprite Handler Released\n";
			}

			//release sound object
			//having trouble finding out how to release this..no release() and can't delete
			/*
			if(dsound!=NULL)
			{dsound->Release();}*/

			//release keyboard objects
			Kill_Keyboard();
			//release mouse objects
			Kill_Mouse();

			//release directinput object
			if(dinput != NULL)
			{dinput->Release();
			cerr<<"DirectInput Object Released\n";}
            
			//shutdown other objects
			Game_End(hWnd);

			//kill program
            PostQuitMessage(0);
            return 0;
    }

    return DefWindowProc( hWnd, msg, wParam, lParam );
}


//sets up windows class
ATOM MyRegisterClass(HINSTANCE hInstance)
{
    //create the window class structure
    WNDCLASSEX wc;
    wc.cbSize = sizeof(WNDCLASSEX); 

    //fill the struct with info
    wc.style         = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc   = (WNDPROC)WinProc;
    wc.cbClsExtra	 = 0;
    wc.cbWndExtra	 = 0;
    wc.hInstance     = hInstance;
    wc.hIcon         = NULL;
    wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
    wc.lpszMenuName  = NULL;
    wc.lpszClassName = APPTITLE;
    wc.hIconSm       = NULL;

    //set up the window with the class info
    return RegisterClassEx(&wc);
}


//entry point for a windows app
int WINAPI WinMain(HINSTANCE hInstance,
                   HINSTANCE hPrevInstance,
                   LPSTR     lpCmdLine,
                   int       nCmdShow)
{
    //Message variable
	MSG msg;

	// register the class
	MyRegisterClass(hInstance);

    // initialize application 
	//main windows handle
    HWND hWnd;

    //create a new window
    hWnd = CreateWindow(
       APPTITLE,              //window class
       APPTITLE,              //title bar
       WS_EX_TOPMOST | WS_VISIBLE | WS_POPUP,   //window style
       CW_USEDEFAULT,         //x position of window
       CW_USEDEFAULT,         //y position of window
       SCREEN_WIDTH,          //width of the window
       SCREEN_HEIGHT,         //height of the window
       NULL,                  //parent window
       NULL,                  //menu
       hInstance,             //application instance
       NULL);                 //window parameters

    //error creating window
    if (!hWnd)
      return FALSE;

    //display the window
    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);
	
	//Logfile initialization
	ofstream LogFile("Log.txt");
	cerr.rdbuf(LogFile.rdbuf( ));
	cerr<<"Log file created successfully!\n\n";

	//some framework specific objects are initialized here and some are
	//initialized in Game_Init();
	//this is to keep repeating framework code out of the main game source

	//initialize direct3d
	if(!Init_Direct3D(hWnd,SCREEN_WIDTH,SCREEN_HEIGHT))
	{	cerr<<"Error Initializing Direct3D\n";
		return 0;}

	//initialize directsound
	if(!Init_DirectSound(hWnd))
	{
		cerr<<"Error Initializing DirectSound\n";
		return 0;
	}

	//initialized directinput (keyboard & mouse)
	if(!Init_DirectInput(hWnd))
	{
		cerr<<"Error Initializing DirectInput\n";
		return 0;
	}
	
	//Initialize mouse
	if(!Init_Mouse(hWnd))
	{
		cerr<<"Error Initializing Mouse\n";
		return 0;
	}

	//intialize keyboard
	if(!Init_Keyboard(hWnd))
	{
		cerr<<"Error Initializing Keyboard\n";
		return 0;
	}

	//initialize the game
    if (!Game_Init(hWnd))
	{cerr<<"Error running Game_Init\n";
		return 0;}

    // main message loop
    int done = 0;
	while (!done)
    {
        if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE)) 
	    {
            //look for quit message
            if (msg.message == WM_QUIT)
                done = 1;

            //decode and pass messages on to WndProc
		    TranslateMessage(&msg);
		    DispatchMessage(&msg);
	    }
		
		//process game loop (else prevents running after window is closed)
        else
		{Game_Run(hWnd);}
    }

	return msg.wParam;
}